var util = require('app').core.require('util'),
    Controller = require('app/lib/Controller');
function respondToUserRequest (req, res, action) {
	return function (err, user) {
		var headers = {'Content-Type': 'application/json'},
		    content = {},
		    //client = req.url.substr(0, 8) === '/client/',
		    status = 200;
		if (err || !user) {
			status = err ? 500 : 400;
			content.failure = true;
			if (err) content.error = err;
			else content.user = {
				_id: guest._id,
				username: guest.username
			};
		} else {
			content.success = true;
			content.session = req.session;
			req.session.user = {
				_id: user._id,
				username: user.username
			};
		}
		if ((req.headers.Accept || req.headers.accept).indexOf('text/html') !== -1) {
			req.session.formResponse = {
				success: content.success,
				failure: content.failure,
				action: 'user/' + action,
				status: status,
				error: content.error,
				user: req.session.user
			};
			headers.Location = /*(client ? '/client' : '') + */'/#/user/' + action;
			status = 302;
		}
		content = JSON.stringify(content);
		headers['Content-Length'] = content.length;
		res.writeHead(status, headers);
		res.end(content);
	};
}
new Controller('User').install(module, function (User) {
	var guest;
	User.find('guest', function (err, user) {
		if (user) guest = user;
		else {
			user = new User.model({username: 'guest'});
			user.save(function (err) {
				if (!err) guest = user;
			});
		}
	});
	this.routes({
		session: function (req, res, next) {
			var session = req.session,
			    content = {};
			if (session.user && session.user._id === guest._id) {
				delete req.session.user;
			}
			if (!req.session.user && guest) {
				req.session.user = {
					_id: guest._id,
					username: guest.username
				};
			}
			content.success = true;
			content.session = session;
			content = JSON.stringify(content);
			res.writeHead(200, {
				'Content-Type': 'application/json',
				'Content-Length': content.length
			});
			res.end(content);
		},
		logout: function (req, res, next) {
			req.session.destroy(function (err) {
				var headers = {'Content-Type': 'application/json'},
				    content = {},
				    status = 200;
				if (err) {
					content.failure = true;
					content.error = err;
					status = 500;
				} else {
					content.success = true;
				}
				content.session = req.session;
				content = JSON.stringify(content);
				headers['Content-Length'] = content.length;
				res.writeHead(status, headers);
				res.end(content);
			});
		},
		login: function (req, res, next) {
			if (!req.body) return next();
			User.login(req.body, respondToUserRequest(req, res, 'login'));
		},
		signup: function (req, res, next) {
			if (!req.body) return next();
			User.signup(req.body, respondToUserRequest(req, res, 'signup'));
		},
		list: function (req, res, next) {
			User.list({}, function (err, users) {
				var headers = {'Content-Type': 'application/json'},
				    content = {},
				    status = 200;
				if (err) {
					content.failure = true;
					content.error = err;
					status = 500;
				} else {
					content.success = true;
					content.users = [];
					users.forEach(function (user) {
						content.users.push({
							_id: user._id,
							username: user.username
						});
					});
				}
				content.session = req.session;
				content = JSON.stringify(content);
				headers['Content-Length'] = content.length;
				res.writeHead(status, headers);
				res.end(content);
			});
		},
		'^[0-9A-Za-z]+$': {
			_name_: 'id', "": function id (req, res, next) {
				if (!req.body) return next();
				req.body._id = req.routeInfo.id;
				User.show(req.body, body, respondToUserRequest(req, res, 'show'));
			},
			save: function (req, res, next) {
				if (!req.body) return next();
				req.body._id = req.routeInfo.id;
				User.save(req.body, respondToUserRequest(req, res, 'save'));
			},
			remove: function (req, res, next) {
				if (!req.body) return next();
				req.body._id = req.routeInfo.id;
				User.remove(req.body, respondToUserRequest(req, res, 'remove'));
			}
		}
	});
});