/*global window require module client*/
(function(){"use strict";
	var util = client.require('util'),
	    Class = client.require('core').require('Class'),
	    Widget = client.require('Widget');
	var Menu,
	    List,
	    Login,
	    Signup;
	module.exports = Class.create('User', Widget, function (obj, supr, klass) {
		obj.newOptions().config({
			id: 'user', names: 'user'
		});
		obj.initUser = function (parent) {
			this.initWidget(parent);
			this.menu = new Menu(this);
			this.list = new List(this);
			this.login = new Login(this);
			this.signup = new Signup(this);
			this.on('parented', function () {
				this.routes({
					login: function () {
						this.user.login.show();
					},
					signup: function () {
						this.user.signup.show();
					},
					logout: function () {
						this.user.logout();
					},
					remove: function () {
						this.user.destroy();
					}
				});
			});
			return this;
		};
		obj.load = function (cb) {
			return supr.load.call(this, function() {
				this.getSession(function () {
					if (!this.isNotGuest()) {
						this.app.goTo('#/user/login');
					} else {
						this.app.goTo('#/');
					}
					util.safeApply(cb, this);
				});
			});
		};
		obj.redraw = function () {
			return supr.redraw.call(this, function (dom) {
				this.signup.hide();
				this.login.hide();
				if (this.isGuest()) {
					this.app.goTo('#/user/login');
				}
			});
		};
		obj.getSession = function (cb) {
			util.ajax('user/session', function (success, json) {
				if (success) {
					this.setSessionFromJSON(json);
					util.safeApply(cb, this, [this.session]);
					this.emit('update session');
				}
			}, this);
			return this;
		};
		obj.setSession = function (session) {
			session = session || {};
			this.session = session;
			this.data = session.user;
			return this;
		};
		obj.setSessionFromJSON = function (json) {
			return this.setSession(JSON.parse(json).session);
		};
		obj.isNotGuest = function () {
			return this.session && this.session.user &&
				this.data.username !== 'guest';
		};
		obj.isGuest = function () {
			return !this.session || !this.session.user ||
				this.data.username === 'guest';
		};
		obj.destroy = function (cb) {
			var url = 'user/' + this.data._id + '/remove';
			util.ajax(url, function (success, json, xhr) {
				try {
					json = JSON.parse(json);
					success = success && json.success;
				} catch (err) {
					success = false;
					json.error = err;
				}
				if (success) {
					this.setSession(json);
					util.safeApply(cb, this);
					this.app.goTo('#/');
				} else {
					
				}
			}, this);
		};
		obj.logout = function (cb) {
			util.ajax('user/logout', function (success, json) {
				if (success) {
					this.setSessionFromJSON(json);
					util.safeApply(cb, this);
					this.redraw();
				}
			}, this);
			return this;
		};
	});
	Menu = require('User/Menu');
	List = require('User/List');
	Login = require('User/Login');
	Signup = require('User/Signup');
}).call(this);