/*global window client module require*/
(function(){"use strict";
	var util = client.require('util'),
	    User = require('../User'),
	    Class = client.require('core').require('Class'),
	    Widget = client.require('Widget');
	module.exports = Class.create('UserSignup', Widget, function (obj, supr, klass) {
		obj.newOptions().config({
			id: 'signup',
			html: ([
				'<form>',
					'<label for="confirmPassword">Confirm Password</label>',
					'<input type="password" name="confirmPassword" value="" id="confirmPassword">',
					'<div class="errors" style="display:none;"></div>',
					'<button type="sumbit">Signup</button>',
					'<a href="#/user/login">Login</a>',
					'<a href="#/">Cancel</a>',
				'</form>'
			]).join(''),
			names: 'signup'
		});
		obj.initUserSignup = function (user) {
			if (!User.check(user)) throw 'Invalid User object.';
			return this.initWidget(this.user = user);
		};
		obj.render = function () {
			return supr.render.call(this, function (dom) {
				dom.root.style.display = 'none';
				dom.form = dom.root.firstChild;
				dom.errors = dom.form.childNodes[2];
				dom.cancel = dom.form.lastChild;
				this.config({parent: this.app.dom.root});
			});
		};
		obj.redraw = function () {
			return supr.redraw.call(this, function (dom) {
				if (this.user.isNotGuest()) {
					dom.cancel.style.display = '';
				} else {
					dom.cancel.style.display = 'none';
				}
			});
		};
		obj.insert = function () {
			return supr.insert.call(this, function (dom) {
				var that = this,
				    events = this.domEvents = {};
				events.signupSubmit = function (e) {
					util.dom.stopEvent(e);
					that.signup();
					return false;
				};
				util.dom.addEvent(dom.form, 'submit', events.signupSubmit);
			});
		};
		obj.remove = function () {
			return supr.remove.call(this, function (dom) {
				var events = this.domEvents;
				util.dom.removeEvent(dom.form, 'submit', events.signupForm);
				delete this.domEvents;
			});
		};
		obj.submit = function (cb) {
			util.ajax('user/signup', function (success, json, xhr) {
				if (success) {
					json = JSON.parse(json);
					this.user.setSession(json.session);
					util.safeApply(cb, this);
					this.hide();
					this.app.goTo('#/');
				} else {

				}
			}, this, {
				method: 'post',
				data: util.dom.formJSON(this.dom.form)
			});
			return this;
		};
		obj.show = function () {
			if (this.isShowing) return this;
			this.isShowing = true;
			this.redraw();
			var login = this.user.login.hide();
			util.dom.insertTop(login.dom.credentials, this.dom.form);
			this.dom.root.style.display = '';
			return this;
		};
		obj.hide = function () {
			if (!this.isShowing) return this;
			delete this.isShowing;
			this.dom.root.style.display = 'none';
			var login = this.user.login;
			util.dom.insertTop(
				login.dom.credentials,
				login.dom.form
			);
			return this;
		};
	});
}).call(this);